#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#include <WiFi.h>
#include <ESPAsyncWebServer.h>
#include <NTPClient.h>
#include <WiFiUdp.h>
#include <RTClib.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "home.h"
#include "street.h"
#include "charts.h"

// ---------------------------------- Параметри Wi-Fi -----------------------------------
const char *ssid = "Redmi";                           // Назва вашої мережі Wi-Fi
const char *password = "123456789";                   // Пароль мережі Wi-Fi
// ------------------------------------------------ -------------------------------------

// ----------------------------- Параметри OpenWeatherMap -------------------------------
const String apiKey = "9c9e29ed0affb48ef48ae09a01017025";   // Ваш API ключ
const String city = "Kremenchuk";                           // Назва міста
// ------------------------------------------------ -------------------------------------

// -------------------------------- Налаштування часу -----------------------------------
int timeZone = 2;                                     // Ваш часовий пояс
unsigned long lastNtpSync = 0;                        //
unsigned long ntpSyncInterval = 5 * 60 * 60 * 1000;   // Інтервал синхронізації (у мілісекундах)
// ------------------------------------------------ -------------------------------------


// Об'єкти для датчиків
Adafruit_BME280 bme; // BME280
RTC_DS3231 rtc; // DS3231

// Веб-сервер
AsyncWebServer server(80);

// NTP
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "pool.ntp.org", timeZone * 3600, 60000);

// Масиви для зберігання даних для графіків (кожен елемент зберігатиме значення за часом)
#define MAX_DATA_POINTS 60

float temperatureData[MAX_DATA_POINTS];
float humidityData[MAX_DATA_POINTS];
float pressureData[MAX_DATA_POINTS];
String timeData[MAX_DATA_POINTS];

// Індекс для запису даних у масив
int dataIndex = 0;

// Оголошення функції для конвертації часу Unix в формат, що читається
String convertUnixTimeToReadable(long unixTime);

// Синхронізація часу з NTP
void syncTimeWithNTP()
{
 if (timeClient.forceUpdate())
 {
 unsigned long epochTime = timeClient.getEpochTime();
 DateTime now(epochTime);
 rtc.adjust(now);
 Serial.println("Час синхронізований з NTP: " + String(now.timestamp().c_str()));
 }
 else
 {
 Serial.println("Помилка синхронізації з сервером NTP.");
 }
}

String getFormattedTime()
{
 DateTime now = rtc.now();
 char timeBuffer[20];
 snprintf(timeBuffer, sizeof(timeBuffer), "%02d:%02d:%02d", now.hour(), now.minute(), now.second());
 return String(timeBuffer);
}

void adjustTimeOffset() {
    DateTime now = rtc.now(); // Отримуємо поточну дату та час
    int month = now.month(); // Отримуємо поточний місяць
    int day = now.day(); // Отримуємо поточний день
    int weekday = now.dayOfTheWeek(); // День тижня (0 = неділя, 6 = субота)

    // Літній час діє з останньої неділі березня до останньої неділі жовтня
    if ((month > 3 && month < 10) || // Якщо місяць між квітнем і вереснем
        (month == 3 && (day - weekday) >= 25) || // Останній тиждень березня
        (month == 10 && (day - weekday) < 25)) { // До останнього тижня жовтня
        timeZone += 1; // Літній час (додаємо годину)
    } else {
        timeZone -= 1; // Зимовий час (віднімаємо годину)
    }
}

// Функція отримання даних з BME280
String getBME280Data()
{
 float temperature = bme.readTemperature();
 float humidity = bme.readHumidity();
 float pressure_mmHg = (bme.readPressure() / 100.0F) * 0.75006157584566; // Тиск у мм рт. ст.

 String data = "Температура: " + String(temperature) + " °C<br>" +
 "Вологість:" + String(humidity) + "%<br>" +
 "Тиск:" + String(pressure_mmHg, 2) + "мм рт. ст.<br>"; // Округлення до 2 знаків
 return data;
}

// Функція для отримання даних про погоду з OpenWeatherMap
String getWeatherData()
{
 HTTPClient http;
 String weatherData = "";
 String url = "http://api.openweathermap.org/data/2.5/weather?q=" + city + "&appid=" + apiKey + "&units=metric&lang=ru";

 http.begin(url);
 int httpCode = http.GET();
 if (httpCode == 200)
 {
 weatherData = http.getString();
 }
 else
 {
 Serial.println("Помилка при отриманні даних погоди:" + String(httpCode));
 }
 http.end();
 return weatherData;
}

// Функція для парсингу даних про погоду
String parseWeatherData(String weatherData)
{
 StaticJsonDocument<512> doc;
 DeserializationError error = deserializeJson(doc, weatherData);
 if (error)
 {
 Serial.println("Помилка при парсингу даних про погоду");
 return "";
 }

 String weatherDescription = doc["weather"][0]["description"].as<String>();
 return weatherDescription;
}

// Функція для парсингу даних про погоду
String parseTempData(String weatherData)
{
 StaticJsonDocument<512> doc;
 DeserializationError error = deserializeJson(doc, weatherData);
 if (error)
 {
 Serial.println("Помилка при парсингу даних про температуру");
 return "";
 }

 String weatherDescription = doc["main"][0]["temp"].as<String>();
 return weatherDescription;
}

// Функція для отримання часу сходу та заходу сонця
String getSunriseSunset(String weatherData)
{
 StaticJsonDocument<512>doc;
 DeserializationError error = deserializeJson(doc, weatherData);
 if (error)
 {
 Serial.println("Помилка при парсингу даних про схід та захід");
 return "";
 }

 long sunrise = doc["sys"]["sunrise"].as<long>();
 long sunset = doc["sys"]["sunset"].as<long>();

 String sunriseStr = convertUnixTimeToReadable(sunrise);
 String sunsetStr = convertUnixTimeToReadable(sunset);

 return sunriseStr;
 return sunsetStr;
}

// Функція для конвертації часу Unix в формат, що читається
String convertUnixTimeToReadable(long unixTime)
{
 time_t t = unixTime;
 struct tm *tmInfo;
 char buffer [20];
 tmInfo = localtime(&t);
 strftime(buffer, 20, "% H: % M: % S", tmInfo);
 return String(buffer);
}

// Функція оновлення даних датчиків
void updateSensorData() {
 // Зчитування даних із датчиків
 float temperature = bme.readTemperature();
 float humidity = bme.readHumidity();
 float pressure = (bme.readPressure() / 100.0F) * 0.75006157584566;
 String currentTime = getFormattedTime(); // Поточний час у форматі рядка

 // Збереження даних у масиви
 temperatureData[dataIndex] = temperature;
 humidityData[dataIndex] = humidity;
 pressureData[dataIndex] = pressure;
 timeData[dataIndex] = currentTime;

 // Інкрементуємо індекс
 dataIndex++;
 if (dataIndex >= MAX_DATA_POINTS) {
 dataIndex = 0; // Перезапуск масиву по колу
 }
}

void setup()
{
 // Налаштування Serial
 Serial.begin(115200);

 // Налаштування I2C та датчиків
 if (!rtc.begin())
 {
 Serial.println("Помилка: модуль DS3231 не знайдено!");
 while (1)
 ;
 }

 if (!bme.begin(0x76))
 {
 Serial.println("Помилка: датчик BME280 не знайдено!");
 while (1)
 ;
 }

 // Підключення до Wi-Fi
 WiFi.begin(ssid, password);
 while (WiFi.status() != WL_CONNECTED)
 {
 delay(1000);
 Serial.println("Підключення до Wi-Fi...");
 }
 Serial.println("Wi-Fi підключений!");
 Serial.println("IP-адреса: " + WiFi.localIP().toString());

 adjustTimeOffset();
 // Синхронізація часу з NTP під час першого запуску
 timeClient.begin();
 syncTimeWithNTP();
 lastNtpSync = millis();

 // Налаштування маршрутів сервера
 server.on("/", HTTP_GET, [](AsyncWebServerRequest *request)
 {
 String html = getHomePage(); // Головна сторінка (Будинок)
 request->send(200, "text/html", html); });

 server.on("/street", HTTP_GET, [](AsyncWebServerRequest *request)
 {
 String html = getStreetPage(); // Сторінка для вулиці
 request->send(200, "text/html", html); });

 server.on("/charts", HTTP_GET, [](AsyncWebServerRequest *request)
 {
 String html = getChartsPage(); // Сторінка для графіків
 request->send(200, "text/html", html); });

 // Ендпоінт для отримання історії
 server.on("/history", HTTP_GET, [](AsyncWebServerRequest *request) {
 String jsonData = "{";
 jsonData += "time":[";

 // Додаємо дані про час
 for (int i = 0; i < MAX_DATA_POINTS; i++) {
 jsonData += "\"" + timeData[i] + "\""; // Перетворимо час на рядок (наприклад, "10:05:23")
 if (i < MAX_DATA_POINTS - 1) jsonData += ",";
 }
 jsonData +="],";

 jsonData += "\"temperature\":[";

 // Додаємо дані про температуру
 for (int i = 0; i < MAX_DATA_POINTS; i++) {
 jsonData += String(temperatureData[i]); // Температура як число
 if (i < MAX_DATA_POINTS - 1) jsonData += ",";
 }
 jsonData +="],";

 jsonData += "humidity":[";

 // Додаємо дані про вологість
 for (int i = 0; i < MAX_DATA_POINTS; i++) {
 jsonData += String(humidityData[i]); // Вологість як число
 if (i < MAX_DATA_POINTS - 1) jsonData += ",";
 }
 jsonData +="],";

 jsonData += "pressure":[";

 // Додаємо дані про тиск
 for (int i = 0; i < MAX_DATA_POINTS; i++) {
 jsonData += String(pressureData[i]); // Тиск як число
 if (i < MAX_DATA_POINTS - 1) jsonData += ",";
 }
 jsonData +="]";

 jsonData +="}";

 // Відправляємо JSON дані у відповідь на запит
 request->send(200, "application/json", jsonData);
});

 server.on("/data", HTTP_GET, [](AsyncWebServerRequest *request)
 {
 String weatherData = getWeatherData();
 String weatherDescription = parseWeatherData(weatherData);

 float temperature = bme.readTemperature();
 float humidity = bme.readHumidity();
 float pressure = bme.readPressure() / 100.0F;
 float pressure_mmHg = pressure * 0.750062;

 String jsonData = "{";
 jsonData += "time": \"" + getFormattedTime() + "\","; // Час лише у форматі HH:MM:SS
 jsonData += "\"temperature\":\"" + String(temperature) + "\",";
 jsonData += "\"humidity\":\"" + String(humidity) + "\",";
 jsonData += "pressure": \"" + String(pressure_mmHg) + "\",";
 jsonData += "weather": \"" + weatherDescription + "\",";
 jsonData += "city": \"" + city + "\"";
 jsonData +="}";

 request->send(200, "application/json", jsonData); });

 server.begin();
}
void loop()
{
 updateSensorData();
 // Періодична синхронізація часу з NTP
 if (millis() - lastNtpSync > ntpSyncInterval)
 {
 syncTimeWithNTP();
 lastNtpSync = millis();
 }
}