#ifndef STREET_H
#define STREET_H

String getStreetPage()
{
    String html = R"rawliteral(
<!DOCTYPE html>
<html lang="ru">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <link rel="preconnect" href="https://fonts.googleapis.com" />
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin />
    <link
      href="https://fonts.googleapis.com/css2?family=Poppins:ital,wght@0,100;0,200;0,300;0,400;0,500;0,600;0,700;0,800;0,900;1,100;1,200;1,300;1,400;1,500;1,600;1,700;1,800;1,900&display=swap"
      rel="stylesheet"
    />
    <title>Умная метеостанция</title>
    <style>
      body {
        font-family: "Poppins", serif;
        margin: 0;
        padding: 0;
        background: rgb(255, 255, 255);
        color: #f5f5f5;
      }
      .link {
        text-decoration: none;
      }
      .container {
        max-width: 90%;
        margin: 0 5% 0 5%;
      }
      .header {
        display: flex;
        justify-content: space-between;
        align-items: center;
      }
      .header strong {
        font-size: 13px;
      }
      #time {
        font-family: "Poppins", serif;
        font-weight: 400;
        font-size: 16px;
        margin-right: 15px;
        color: rgb(27, 27, 27);
      }
      .data-section {
        display: flex;
        flex-wrap: wrap;
        justify-content: space-between;
      }

      .data-box {
        flex: 1 1 calc(45% - 10px);
        background: #f1f1f1;
        margin: 5px;
        padding: 10px 25px 20px 10px;
        border-radius: 8px;
        border: 1px solid #e4e4e4;
        box-sizing: border-box;
      }

      .data-box strong {
        font-family: "Poppins", serif;
        font-weight: 600;
        display: block;
        text-align: left;
        font-size: 15px;
        margin-bottom: 25px;
        color: #1e9aff;
      }

      .data-box span {
        font-family: "Poppins", serif;
        font-weight: 650;
        color: rgb(27, 27, 27);
        font-size: 22px;
      }
      .nav-bar {
        position: fixed;
        bottom: 0;
        left: 0;
        right: 0;
        background: rgb(245, 245, 245);
        display: flex;
        justify-content: space-around;
        align-items: center;
        height: 60px;
        z-index: 100;
      }
      .nav-item {
        text-decoration: none;
        font-family: "Poppins", serif;
        font-weight: 550;
        text-align: center;
        font-size: 14px;
        color: rgb(27, 27, 27);
        cursor: pointer;
        transition: color 0.3s ease;
      }
      .nav-item:hover {
        color: #ffd700;
      }
      .footer {
        text-align: center;
        font-size: 14px;
        color: #888;
        margin-top: 20px;
      }
      .city-name {
        font-family: "Poppins", serif;
        font-weight: 600;
        color: rgb(27, 27, 27);
        margin: 15px 15px 15px 15px;
        text-align: left;
        font-size: 24px;
      }
    </style>
    <script>
      function updateData() {
        fetch("/data")
          .then((response) => response.json())
          .then((data) => {
            document.getElementById("time").innerText = data.time;
            document.getElementById("weather").innerText = data.weather;
            document.getElementById("city").innerText = data.city;
            document.getElementById("sunrise").innerText = data.sunrise;
            document.getElementById("sunset").innerText = data.sunset;
          })
          .catch((error) => console.error("Помилка завантаження даних:", error));
      }

      setInterval(updateData, 1000);
      window.onload = updateData;
    </script>
  </head>
  <body>
    <div class="header">
      <div class="city-name" id="city">Loading...</div>
      <span id="time">Loading...</span>
    </div>
    <div class="container">
      <div class="data-section">
        <div class="data-box">
          <strong>Weather</strong>
          <span id="weather">Loading...</span>
        </div>
        <div class="data-box">
          <strong>Sunrise</strong>
          <span id="sunrise">Loading...</span>
        </div>
        <div class="data-box">
          <strong>Sunset</strong>
          <span id="sunset">Loading...</span>
        </div>
      </div>
    </div>

    <div class="nav-bar">
      <a class="nav-item" href="/">Houme</a>
      <a class="nav-item" href="/street">Street</a>
    </div>
  </body>
</html>

    )rawliteral";
    return html;
}

#endif